/* This file was generated by syncqt. */
#ifndef QT_QT3DINPUT_VERSION_H
#define QT_QT3DINPUT_VERSION_H

#define QT3DINPUT_VERSION_STR "5.15.13"

#define QT3DINPUT_VERSION 0x050F0D

#endif // QT_QT3DINPUT_VERSION_H
